//
// To compile in Visual Studio 2022 (free Community Edition): 
//   1) Create a new Dynamic Link Library project
//   2) Copy this .c file into the Project Directory that was created in #1 above
//   3) Remove all other Source Files added by #1 above (dllmain.cpp, psh.h, etc.)
//   4) Use Project -> Add Existing Item to select and add this .c file to the Source Files
//   5) Change Active Solution Platform from x64 to x86 in Configuration Manager
//   6) turn off the Use Precompiled Headers option in Project -> "ProjectName" Properties -> C/C++ -> Precompiled Heading section -> Precompiled Header -> Not Using Precompiled Headers
//   7) Choose Build Solution
//   8) .dll file compiled into \debug project directory will need to be copied to C:\NeuroShell Trader\Template directory
//      for the .dll file to be seen/used by a NeuroShell Trader chart.
//

#include <windows.h>   
#include <math.h>   

const float MISSING = (float) 3.4e38;    // NeuroShell Trader missing value constant

double calcAngle(float *Close, long Period, long CurrentBar, double Angle1 )
{
	double Sx = 0.0, Sy = 0.0, Sxx = 0.0, Sxy = 0.0, Syy = 0.0;
	double X = 0.0, Y = 0.0, Real = 0.0, Imag = 0.0;
	double Angle = 0.0;
	long count = 0;

	double radiansconvertmult;
	radiansconvertmult = 3.1415926535 / 180.0;


	//Correlate with Cosine wave having a fixed period
	Sx = 0;
	Sy = 0;
	Sxx = 0;
	Sxy = 0;
	Syy = 0;
	for (count = 1; count <= Period; count++)
	{
		X = Close[CurrentBar - (count - 1)];
		Y = cos(radiansconvertmult * 360 * (count - 1) / Period);
		Sx = Sx + X;
		Sy = Sy + Y;
		Sxx = Sxx + X * X;
		Sxy = Sxy + X * Y;
		Syy = Syy + Y * Y;
	}

	if ((Period * Sxx - Sx * Sx > 0) && (Period * Syy - Sy * Sy > 0))
		Real = (Period * Sxy - Sx * Sy) / sqrt((Period * Sxx - Sx * Sx) * (Period * Syy - Sy * Sy));

	//Correlate with a Negative Sine wave having a fixed period
	Sx = 0;
	Sy = 0;
	Sxx = 0;
	Sxy = 0;
	Syy = 0;
	for (count = 1; count <= Period; count++)
	{
		X = Close[CurrentBar - (count - 1)];
		Y = -sin(radiansconvertmult * 360 * (count - 1) / Period);
		Sx = Sx + X;
		Sy = Sy + Y;
		Sxx = Sxx + X * X;
		Sxy = Sxy + X * Y;
		Syy = Syy + Y * Y;
	}

	if ((Period * Sxx - Sx * Sx) > 0 && (Period * Syy - Sy * Sy) > 0)
		Imag = (Period * Sxy - Sx * Sy) / sqrt((Period * Sxx - Sx * Sx) * (Period * Syy - Sy * Sy));

	//Compute the angle as an arctangent function and resolve ambiguity

	if (Real != 0) Angle = 90 - atan(Imag / Real) / radiansconvertmult;
	if (Real < 0) Angle = Angle - 180;

	//compensate for angle wraparound
	if (((fabs(Angle1) - fabs(Angle - 360)) < (Angle - Angle1)) && (Angle > 90) && (Angle1 < -90)) Angle = Angle - 360;

	//angle cannot go backwards
	if (Angle < Angle1 && ((Angle > -135 && Angle1 < 135) || (Angle < -90 && Angle1 < -90))) Angle = Angle1;

	return Angle;
}

_declspec(dllexport) void PhasorIndicator(float* Close, long Period, long numrows, float* output)
{
	// Phasor Analysis
	// (C) 2013 - 2022 John F.Ehlers

	double Angle = 0.0, Angle1 = 0.0;
	long CurrentBar = 0;

	for (CurrentBar = 0; CurrentBar < numrows; CurrentBar++)
	{	
		if (CurrentBar > Period)
		{
			Angle = calcAngle(Close, Period, CurrentBar, Angle1);

			//Phasor Indicator
			output[CurrentBar] = (float)Angle;    //Plot1(Angle, "Angle", red, 4, 4);

			Angle1 = Angle;
		}
	}
}

_declspec(dllexport) void CyclePeriods(float* Close, long Period, long numrows, float* output)
{
	// Phasor Analysis
	// (C) 2013 - 2022 John F.Ehlers

	double Angle = 0.0, Angle1 = 0.0;
	long DerivedPeriod = 0, CurrentBar = 0;
	double DeltaAngle = 0.0, DeltaAngle1 = 0.0, AvgPeriod = 0.0;
	long State = 0;

	for (CurrentBar = 0; CurrentBar < numrows; CurrentBar++)
	{
		if (CurrentBar > Period)
		{
			Angle = calcAngle(Close, Period, CurrentBar, Angle1);

			//Frequency derived from rate-change of phase
			DeltaAngle = Angle - Angle1;
			if ( DeltaAngle <= 0 ) DeltaAngle = DeltaAngle1;
			if ( DeltaAngle != 0 ) DerivedPeriod = 360 / DeltaAngle;
			if ( DerivedPeriod > 60 ) DerivedPeriod = 60;

			output[CurrentBar] = (float) DerivedPeriod;    //Plot9(DerivedPeriod, "", red, 4, 4);

			Angle1 = Angle;
			DeltaAngle1 = DeltaAngle;
		}
	}
}

_declspec(dllexport) void TrendState(float* Close, long Period, long numrows, float* output)
{
	// Phasor Analysis
	// (C) 2013 - 2022 John F.Ehlers

	double Angle = 0.0, Angle1 = 0.0;
	long CurrentBar = 0;
	long State = 0;

	for (CurrentBar = 0; CurrentBar < numrows; CurrentBar++)
	{
		if (CurrentBar > Period)
		{
			Angle = calcAngle(Close, Period, CurrentBar, Angle1);

			//Trend State Variable
			State = 0;
			if (Angle - Angle1 <= 6)
			{	if (Angle >= 90 || Angle <= -90) State = 1;
				if (Angle > -90 && Angle < 90) State = -1;
			}
			output[CurrentBar] = (float) State;    //Plot10(State, "", red, 4, 4);

			Angle1 = Angle;
		}
	}
}